<?php
/*
Plugin Name: Term Description
Plugin URI: http://localhost
Description: Этот плагин ищет заданные термины в постах и превращает их в ссылки на соответствующие страницы.
Version: 1.0
Author: Vladimir Statsenko
Author URI: http://www.simplecoding.org
License: GPL2
*/

/*
 После активации плагин создает в админке (в разделе Tools) страницу с формой
 создания новых терминов и управления существующими.
 
 Терминами могут быть любые выражения, содержащие одно или несколько слов.
 Каждому термину назначается ссылка на страницу с пояснениями. Эти страницы
 должны быть созданы заранее.
 
 При формировании постов, плагин ищет в них указанные термины и преобразовывает
 их в ссылки.
 
 Термины игнорируются в следующих случаях.
 1) Если они уже находятся внутри ссылки.
 2) Если они находятся внутри тега (например, в атрибуте alt тега img).
 3) Если они находятся внутри заголовков (теги h1 - h6).
 4) Если они являются составляющей частью другого слова (например, термин
 рот, найденное слово - крот).
 
 Кроме того, вы можете указать где заменять термины, в тексте поста и/или
 комментариях.
*/

class TermDescription {
    /**
     * Этот метод выполняет обработку запросов со страницы настроек
     */
    public function managementHandler() {
        //загружаем файл переводов (.mo)
        //они должны называться termdescription-locale.mo
        $plugin_dir = basename(dirname(__FILE__));
		$domain = TermDescription::getTextDomain();
		load_plugin_textdomain( $domain, '/wp-content/plugins/'.$plugin_dir, $plugin_dir );

		$parentFile = 'tools.php?page=' . TermDescription::getPluginSlug();
        
        //если запрос пришел со страницы плагина
		if ( 'termdescription.php' == substr( $parentFile, -19 ) ) {
            //получаем список терминов
			$terms = get_option( 'td_terms' );
            
            //читаем название команды
            $action = $_GET['action'];
            
            switch ($action) {
                //добавляем термин
                case 'add':
					check_admin_referer('td-add');
					$term = $_GET['term'];
					$pageid = $_GET['termpageid'];
                    //сохраняем термин
					$key = TermDescription::getNewKey($terms);
					$page = get_page($pageid);
					$terms[$key] = array(
							'term'=>$term,
							'pageid'=>$pageid,
							'url'=>get_page_link($page->ID),
							'title'=>$page->post_title,
						);
					update_option('td_terms', $terms);
					$message = 1;
	  	  			wp_redirect( "$parentFile&m=$message" );
                    break;
                //удаляем термин
				case 'delete':
					check_admin_referer('delete_term');
					if (isset($_GET['termid']) && is_numeric($_GET['termid'])
							&& isset($terms[$_GET['termid']])) {
						unset($terms[$_GET['termid']]);
						update_option('td_terms', $terms);
						$message = 2;
					}
					else {
						$message = 5;
					}
					wp_redirect("$parentFile&m=$message");
					break;
                //изменяем термин
                case 'edit':
					check_admin_referer('td-edit');
                    if (isset($_GET['termid']) && is_numeric($_GET['termid'])
                            && isset($terms[$_GET['termid']])) {
                        $term = $_GET['term'];
                        $pageid = $_GET['termpageid'];
                        $page = get_page($pageid);
                        $terms[$_GET['termid']] = array(
							'term'=>$term,
							'pageid'=>$pageid,
							'url'=>get_page_link($page->ID),
							'title'=>$page->post_title,
                        );
                        update_option('td_terms', $terms);
                        $message = 3;
                    }
                    else {
                        $message = 6;
                    }
					wp_redirect("$parentFile&m=$message");
					break;
            }
        }
    }
    
    /**
     * Возвращает $menu_slug плагина (задается в функции add_management_page)
     * (termdescription/termdescription.php)
     */
    public function getPluginSlug() {
        return $_GET['page'];
    }
    
    /**
     * Возвращает уникальный идентификатор текстового домена для локализации
     */
    public function getTextDomain() {
        return 'termdescription';
    }
    
	/**
	 * Возвращает id нового термина
	 */
	public function getNewKey($terms) {
		//определяем максимальный существующий id в массиве и возвращаем id+1,
        //если термины еще не созданы, возвращаем 1
		if (false == $terms) {
			return 1;
		}
		else {
			krsort($terms, SORT_NUMERIC);
			return (int)key($terms) + 1;
		}
	}
	
    /**
     * Этот метод добавляет в админку страницу с настройками плагина
     */
    public function addPages() {
        $domain = TermDescription::getTextDomain();
        
        add_management_page( __('Term Description', $domain), __('Term Description', $domain), "manage_options", __FILE__, array('TermDescription', 'manageTerms'));
    }
    
    /**
     * Этот метод создаёт страницу с настройками плагина
     */
    public function manageTerms() {
        $domain = TermDescription::getTextDomain();
        //на эту страницу редиректят все обработчики команд, и при этом они
        //передают свои сообщения, которые нужно вывести
        if (isset($_GET['updated'])) {
            echo '<div id="message" class="updated fade"><p>'.__('Options saved', $domain).'</p></div>';
        }
        else if (isset($_GET['m']) && is_numeric($_GET['m'])) {
            $message = $_GET['m'];
            $messages[1] = __('Term created', $domain);
            $messages[2] = __('Term deleted', $domain);
            $messages[3] = __('Term updated', $domain);
            $messages[4] = __('Term create error', $domain);
            $messages[5] = __('Term delete error', $domain);
            $messages[6] = __('Term update error', $domain);
            echo '<div id="message" class="updated fade"><p>'.$messages[$message].'</p></div>';
        }
        
		$terms = get_option('td_terms');
		
        //дальше идет разметка страницы
?>
<div class="wrap">
    <div id="icon-edit" class="icon32"><br/></div>
    <h2><?php _e('Terms descriptions plugin', $domain); ?></h2>
    <br class="clear" />

    <div id="col-container">
        <div id="col-right">
			<div class="col-wrap">
			<?php TermDescription::showTerms($terms); ?>
			</div>
        </div>
        <div id="col-left">
            <div class="col-wrap">
                <?php
                //выводим форму создания или изменения термина
                if ('term_edit' == $_GET['action'] && !empty($_GET['termid'])
                        && is_numeric($_GET['termid'])) {
                    TermDescription::showTermForm($_GET['termid'], $terms[$_GET['termid']]);
                }
                else {
                    TermDescription::showTermForm();
                }
                ?>
            </div>
        </div>

    </div>
</div>
<?php
    }
    
    /**
     * Создаёт разметку форму добавления нового термина или редактирования
     * существующего и форму с настройками
     */
    public function showTermForm($termid = '', $term = null) {
        $domain = TermDescription::getTextDomain();
        
        //если нужно показать форму изменения термина
        if (!empty($termid) && isset($term)) {
            $formTitle = __('Update term', $domain);
            $submitText = __('Update', $domain);
            $formOpenTag = '<form name="editterm" id="editterm" method="get" action="" class="validate">';
            $action = 'edit';
            $nonse = 'td-edit';
            $termidField = '<input type="hidden" id="termid" name="termid" value="'.$termid.'" />';
        }
        else {
            $formTitle = __('Create term', $domain);
            $submitText = __('Create', $domain);
            $formOpenTag = '<form name="addterm" id="addterm" method="get" action="" class="validate">';
            $action = 'add';
            $nonse = 'td-add';
            $termidField = '';
        }
?>
<div class="form-wrap">
	<h3><?php echo $formTitle; ?></h3>
	<div id="ajax-response"></div>
	<?php echo $formOpenTag; ?>
		<input type="hidden" name="page" value="<?php echo TermDescription::getPluginSlug() ?>"/>
		<input type="hidden" name="action" value="<?php echo $action ?>" />
        <?php echo $termidField; ?>
<?php wp_original_referer_field(true, 'previous'); wp_nonce_field($nonse); ?>
		<div class="form-field form-required">
			<label for="term"><?php _e('Term', $domain); ?></label>
			<input name="term" id="term" type="text" value="<?php echo (isset($term)) ? $term['term'] : ''; ?>" size="20" />
			<p><?php _e('Term can contain one or several words and will be converted to a link', $domain); ?>.</p>
		</div>
		<div class="form-field form-required">
			<label for="termpageid"><?php _e('Link to page', $domain); ?></label>
            <select name="termpageid" id="termpageid">
            <?php
                $pages = get_pages();
                foreach ($pages as $page) {
                    $selected = '';
                    if ($page->ID == $term['pageid']) {
                        $selected = 'selected="selected"';
                    }
                    $option = '<option value="'.$page->ID.'" '.$selected.'>';
                    $option .= $page->post_title;
                    $option .= '</option>';
                    echo $option;
                }
            ?>
            </select>
			<p><?php _e('Select a page you want to link to', $domain); ?>.</p>
		</div>
		<p class="submit">
			<?php if ( 'edit' == $action ) echo '<a accesskey="c" title="'.__('Cancel', $domain).'" class="cancel button-secondary alignright" href="tools.php?page=' . TermDescription::getPluginSlug() . '">'.__('Cancel', $domain).'</a>' ?>
			<input type="submit" class="button-primary alignleft" name="submit" value="<?php echo $submitText; ?>" />
		</p>
	</form>
</div>
<div class="form-wrap">
	<h3><?php _e('Options', $domain); ?></h3>
    <form action="options.php" method="post" name="showon" id="showon">
        <?php settings_fields( 'td-settings-group' ); ?>
        <table class="form-table">
            <tbody>
            <tr valign="top">
                <th scope="row"><?php _e('Convert terms to links', $domain); ?></th>
                <td>
                <fieldset>
                    <legend class="screen-reader-text"><span><?php _e('Convert terms to links', $domain); ?></span></legend>
                    <?php
                        $showon = get_option('td_target');
                        $tPosts = $tComments = '';
                        if (is_array($showon)) {
                            $tPosts = (in_array('posts', $showon)) ? 'checked="checked"' : '';
                            $tComments = (in_array('comments', $showon)) ? 'checked="checked"' : '';
                        }
                    ?>
                    <label><input type="checkbox" value="posts" name="td_target[]" <?php echo $tPosts; ?> /> <?php _e('in posts', $domain); ?></label><br />
                    <label><input type="checkbox" value="comments" name="td_target[]" <?php echo $tComments; ?> /> <?php _e('in comments', $domain); ?></label>
                </fieldset>
                </td>
            </tr>
            </tbody>
        </table>
        <p class="submit">
			<input type="submit" class="button-primary alignleft" name="submit" value="<?php _e('Save', $domain); ?>" />
        </p>
    </form>
</div>
<?php
    }
	
	/**
	 * Формирует страницу со списком терминов
	 */
	public function showTerms($terms) {
        $domain = TermDescription::getTextDomain();
        
		if (false == $terms) {
			echo '<strong>'.__('Terms not found', $domain).'</strong>';
		}
		else {
?>
<table class="widefat">
	<thead>
	<tr>
		<th scope="col"><?php _e('Term', $domain); ?></th>
		<th scope="col"><?php _e('Page', $domain); ?></th>
	</tr>
	</thead>
	<tfoot>
	<tr>
		<th scope="col"><?php _e('Term', $domain); ?></th>
		<th scope="col"><?php _e('Page', $domain); ?></th>
	</tr>
	</tfoot>
	<tbody id="the-list">
	<?php foreach ($terms as $key => $term) { ?>
		<tr class="iedit alternate">
			<td class="name column-name">
				<span style="display:block"><strong><?php echo $term['term']; ?></strong></span>
				<div class="row-actions">
					<span class="edit"><a href="tools.php?page=<?php echo TermDescription::getPluginSlug(); ?>&amp;action=term_edit&amp;termid=<?php echo $key; ?>"><?php _e('Edit', $domain); ?></a></span> |
				  	<span class="delete">
				  		<?php
				  			$link = 'tools.php?page=' . TermDescription::getPluginSlug() . '&amp;action=delete&amp;termid=' . $key;
				  			$link = ( function_exists( 'wp_nonce_url' ) ) ? wp_nonce_url( $link, 'delete_term' ) : $link;
				  		?>
				  		<a class="submitdelete" href="<?php echo $link ?>" onclick="if ( confirm('<?php _e('Are you sure?', $domain); ?>') ) { return true;}return false;">
				  			<?php _e('Delete', $domain); ?>
				  		</a>
				  	</span>
				</div>
			</td>
			<td class="name column-name">
				<a href="<?php echo $term['url']; ?>"><?php echo $term['title']; ?></a>
			</td>
		</tr>
	<?php } ?>
	</tbody>
</table>
<?php
		}
	}
    
    /**
     * Регистрирует группу чекбоксов (в форме настроек)
     */
    public function registerTdSettings() {
        register_setting( 'td-settings-group', 'td_target' );
    }
    
    /**
     * Заменяет термины на ссылки
     */
    public function termReplace($content) {
		//если отображается страница - не делаем ничего
		if (is_page()) {
			return $content;
		}
        $terms = get_option('td_terms');
        if ($terms) {
            foreach ($terms as $term) {
                //нужно пропустить термины, которые находятся внутри ссылок, заголовков (теги h1..h6)
                //или внутри атрибутов тегов
                /*
                 Алгоритм:
                 1) находим все ссылки и теги
                 2) разбиваем текст по ним
                 3) заменяем термины в оставшихся фрагментах
                 4) учитываем, что перед или термина может стоять скобка, знак препинания и т.п. 
                */
                preg_match_all('/<a.*?>.*?<\/a>|<h\d.*?\/h\d>|<.*?>/isu', $content,
                    $matches, PREG_OFFSET_CAPTURE);
                $result = '';
                $startPos = 0;
                foreach ($matches[0] as $match) {
                    //был ли текст до этого совпадания
                    $length = $match[1] - $startPos;
                    if ($length > 0) {
                        //ищем термин
                        $text = substr($content, $startPos, $length);
                        $result .= preg_replace('/([\s\r\n\:\;\!\?\.\,\)\(<>]{1}|^)('.$term['term'].')([\s\r\n\:\;\!\?\.\,\)\(<>]{1}|$)/isu', '$1<a href="'.$term['url'].'">$2</a>$3', $text);
                    }
                    //добавляем к результату сам тег
                    $result .= $match[0];
                    $startPos = $match[1] + strlen($match[0]);
                }
                //проверяем, обработан ли весь текст
                //(на случай, если $content не содержит закрывающего тега в конце)
                if ($startPos < strlen($content)) {
                        $text = substr($content, $startPos);
                        $result .= preg_replace('/([\s\r\n\:\;\!\?\.\,\)\(<>]{1}|^)('.$term['term'].')([\s\r\n\:\;\!\?\.\,\)\(<>]{1}|$)/isu', '$1<a href="'.$term['url'].'">$2</a>$3', $text);
                }
				$content = $result;
            }
        }
        return $content;
    }
}

//обработчик запросов
add_action( 'admin_init', array( 'TermDescription', 'managementHandler' ) );
//обработчик формы с настройками
add_action( 'admin_init', array( 'TermDescription', 'registerTdSettings' ) );

//инсталляция плагина, на данный момент не используется
//register_activation_hook( __FILE__, array( 'TermDescription', 'install' ) );
//деинсталляция плагина, на данный момент не используется
//register_deactivation_hook( __FILE__, array( 'TermDescription', 'uninstall' ) );

//добавляем фильтры для постов и комментариев в которых нужно преобразовывать ссылки в термины
$tdTarget = get_option( 'td_target' );
if (is_array($tdTarget) && in_array('posts', $tdTarget)) {
	add_filter( 'the_content', array( 'TermDescription', 'termReplace' ) );
}
if (is_array($tdTarget) && in_array('comments', $tdTarget))
	add_filter( 'comment_text', array( 'TermDescription', 'termReplace' ) );
  
//создание страницы с настройками
add_action( 'admin_menu', array( 'TermDescription', 'addPages' ) );

//end of termdescription.php